@@ -21,6 +21,7 @@
 #include "../PitchName.h"
 #include "../Spectrum.h"
 #include "../WaveTrack.h"
+#include "TimeWarper.h"
 
 #include <math.h>
 
@@ -70,7 +71,7 @@ void EffectChangePitch::DeduceFrequencies()
 {
    // As a neat trick, attempt to get the frequency of the note at the
    // beginning of the selection.
-   TrackListIterator iter(mWaveTracks);
+   SelectedTrackListOfKindIterator iter(Track::Wave, mTracks);
    WaveTrack *track = (WaveTrack *) iter.First();
    if (track) {
       const int windowSize = 1024;
@@ -105,9 +106,9 @@ void EffectChangePitch::DeduceFrequencies()
       m_ToFrequency = (m_FromFrequency * (100.0 + m_PercentChange)) / 100.0;
 
       // Now we can set the pitch control values. 
-      m_FromPitchIndex = PitchIndex(Freq2Pitch(m_FromFrequency));
+      m_FromPitchIndex = PitchIndex(FreqToMIDInoteNumber(m_FromFrequency));
       m_bWantPitchDown = (m_ToFrequency < m_FromFrequency);
-      m_ToPitchIndex = PitchIndex(Freq2Pitch(m_ToFrequency));
+      m_ToPitchIndex = PitchIndex(FreqToMIDInoteNumber(m_ToFrequency));
    }
 }
 
@@ -153,6 +154,7 @@ bool EffectChangePitch::TransferParameters( Shuttle & shuttle )
 bool EffectChangePitch::Process()
 {
    mSoundTouch = new SoundTouch();
+   SetTimeWarper(new IdentityTimeWarper());
    mSoundTouch->setPitchSemiTones((float)(m_SemitonesChange));
    return this->EffectSoundTouch::Process();
 }
@@ -588,7 +590,7 @@ void ChangePitchDialog::OnText_FromFrequency(wxCommandEvent & event)
       str.ToDouble(&newDouble);
       m_FromFrequency = newDouble;
 
-      m_FromPitchIndex = PitchIndex(Freq2Pitch(m_FromFrequency));
+      m_FromPitchIndex = PitchIndex(FreqToMIDInoteNumber(m_FromFrequency));
       this->Calc_ToFrequency();
       m_bWantPitchDown = (m_ToFrequency < m_FromFrequency);
       this->Calc_ToPitchIndex(); // Call *after* m_bWantPitchDown is updated.
