@@ -6,22 +6,37 @@
 
   Vaughan Johnson, Dominic Mazzoni
   
-  Change Speed effect, that affects both pitch & tempo.
+*******************************************************************//**
 
-**********************************************************************/
+\class EffectChangeSpeed
+\brief An Effect that affects both pitch & Speed.
 
-#include "../Audacity.h" // for USE_LIBSAMPLERATE
+*//****************************************************************//**
+
+\class ChangeSpeedDialog
+\brief Dialog used with EffectChangeSpeed
+
+*//*******************************************************************/
+
+
+#include "../Audacity.h"
+
+#include "ChangeSpeed.h"
+
+#include "../ShuttleGui.h"
+#include "../Envelope.h"
+#include "../Prefs.h"
+#include "../Project.h"
+#include "TimeWarper.h"
 
 #include <math.h>
 
-#include <wx/intl.h>
+#include <wx/button.h>
 #include <wx/msgdlg.h> // for wxMessageBox
+#include <wx/sizer.h>
+#include <wx/stattext.h>
 #include <wx/valtext.h>
 
-#include "../Envelope.h"
-#include "../Prefs.h"
-#include "../Internat.h"
-#include "ChangeSpeed.h"
 
 //
 // EffectChangeSpeed
@@ -38,13 +53,13 @@ EffectChangeSpeed::EffectChangeSpeed()
 wxString EffectChangeSpeed::GetEffectDescription() { 
    // Note: This is useful only after change amount has been set. 
    return wxString::Format(_("Applied effect: %s %.1f%%"), 
-                           (const char *)(this->GetEffectName()), 
+                           this->GetEffectName().c_str(), 
 									m_PercentChange); 
 } 
 
 bool EffectChangeSpeed::PromptUser()
 {
-   ChangeSpeedDialog dlog(this, mParent, -1, _("Change Speed"));
+   ChangeSpeedDialog dlog(this, mParent);
    dlog.m_PercentChange = m_PercentChange;
    dlog.m_FromVinyl = m_FromVinyl;
    dlog.m_ToVinyl = m_ToVinyl;
@@ -55,7 +70,7 @@ bool EffectChangeSpeed::PromptUser()
    dlog.CentreOnParent();
    dlog.ShowModal();
 
-   if (!dlog.GetReturnCode())
+   if (dlog.GetReturnCode() == wxID_CANCEL)
       return false;
 
    m_PercentChange = dlog.m_PercentChange;
@@ -64,53 +79,94 @@ bool EffectChangeSpeed::PromptUser()
 
    return true;
 }
+bool EffectChangeSpeed::TransferParameters( Shuttle & shuttle )
+{  
+   shuttle.TransferDouble(wxT("Percentage"),m_PercentChange,0.0);
+   return true;
+}
 
 bool EffectChangeSpeed::Process()
 {
 	// Similar to EffectSoundTouch::Process()
 
    //Iterate over each track
-   TrackListIterator iter(mWaveTracks);
-   WaveTrack *track = (WaveTrack *) iter.First();
+   //Track::All is needed because this effect needs to introduce silence in the group tracks to keep sync
+   this->CopyInputTracks(Track::All); // Set up mOutputTracks.
+   bool bGoodResult = true;
+
+   TrackListIterator iter(mOutputTracks);
+   // go to first wavetrack
+   Track* t;
+   for (t = iter.First(); t->GetKind() != Track::Wave; t = iter.Next());
+   if (!t)
+      return false;
+   WaveTrack* pOutWaveTrack = (WaveTrack*)t;
    mCurTrackNum = 0;
 	m_maxNewLength = 0.0;
-	double curT0;
-	double curT1;
-   while (track) {
-      //Get start and end times from track
-      double trackStart = track->GetStartTime();
-      double trackEnd = track->GetEndTime();
+	
+   //Get start and end times from track
+   mCurT0 = pOutWaveTrack->GetStartTime();
+   mCurT1 = pOutWaveTrack->GetEndTime();
 
-      //Set the current bounds to whichever left marker is
-      //greater and whichever right marker is less:
-      curT0 = mT0 < trackStart? trackStart: mT0;
-      curT1 = mT1 > trackEnd? trackEnd: mT1;
+   //Set the current bounds to whichever left marker is
+   //greater and whichever right marker is less:
+   mCurT0 = wxMax(mT0, mCurT0);
+   mCurT1 = wxMin(mT1, mCurT1);
 
-      // Process only if the right marker is to the right of the left marker
-      if (curT1 > curT0) {
+   double len = pOutWaveTrack->GetEndTime() - pOutWaveTrack->GetStartTime();
 
-         //Transform the marker timepoints to samples
-         longSampleCount start = track->TimeToLongSamples(curT0);
-         longSampleCount end = track->TimeToLongSamples(curT1);
+   // we only do a "group change" in the first selected track of the group. 
+   // ClearAndPaste has a call to Paste that does changes to the group tracks
+   bool first = true;
 
-         //ProcessOne() (implemented below) processes a single track
-         if (!ProcessOne(track, start, end))
-            return false;
+   while (t != NULL)
+   {
+      if (t->GetKind() == Track::Label)
+         first = true;
+      else if (t->GetKind() == Track::Wave && t->GetSelected()) {
+         pOutWaveTrack = (WaveTrack*)t;
+         //Get start and end times from track
+         mCurT0 = pOutWaveTrack->GetStartTime();
+         mCurT1 = pOutWaveTrack->GetEndTime();
+
+         //Set the current bounds to whichever left marker is
+         //greater and whichever right marker is less:
+         mCurT0 = wxMax(mT0, mCurT0);
+         mCurT1 = wxMin(mT1, mCurT1);
+
+         // Process only if the right marker is to the right of the left marker
+         if (mCurT1 > mCurT0) {       
+            //Transform the marker timepoints to samples
+            sampleCount start = pOutWaveTrack->TimeToLongSamples(mCurT0);
+            sampleCount end = pOutWaveTrack->TimeToLongSamples(mCurT1);
+
+            //ProcessOne() (implemented below) processes a single track
+            if (!ProcessOne(pOutWaveTrack, start, end, first))
+            {
+               bGoodResult = false;
+               break;
+            }
+            first = false;
+         }
+         mCurTrackNum++;
       }
       
       //Iterate to the next track
-      track = (WaveTrack *) iter.Next();
-      mCurTrackNum++;
+      t=iter.Next();
    }
 
-	mT1 = mT0 + m_maxNewLength; // Update selection.
-   return true;
+   if (bGoodResult)
+      ReplaceProcessedTracks(bGoodResult);
+
+   mT1 = mT0 + m_maxNewLength; // Update selection.
+
+   return bGoodResult;
 }
 
 // ProcessOne() takes a track, transforms it to bunch of buffer-blocks,
 // and calls libsamplerate code on these blocks.
 bool EffectChangeSpeed::ProcessOne(WaveTrack * track,
-												longSampleCount start, longSampleCount end)
+									sampleCount start, sampleCount end, bool first)
 {
 	if (track == NULL)
 		return false;
@@ -118,7 +174,8 @@ bool EffectChangeSpeed::ProcessOne(WaveTrack * track,
 	// initialization, per examples of Mixer::Mixer and
    // EffectSoundTouch::ProcessOne
 
-   WaveTrack * outputTrack = mFactory->NewWaveTrack(track->GetSampleFormat());
+   WaveTrack * outputTrack = mFactory->NewWaveTrack(track->GetSampleFormat(),
+                                                    track->GetRate());
 
    //Get the length of the selection (as double). len is
    //used simple to calculate a progress meter, so it is easier
@@ -143,7 +200,7 @@ bool EffectChangeSpeed::ProcessOne(WaveTrack * track,
    //sample the current buffer starts at.
 	bool bLoopSuccess = true;
    sampleCount blockSize;
-  	longSampleCount samplePos = start;
+  	sampleCount samplePos = start;
    while (samplePos < end) {
       //Get a blockSize of samples (smaller than the size of the buffer)
       blockSize = track->GetBestBlockSize(samplePos);
@@ -192,8 +249,16 @@ bool EffectChangeSpeed::ProcessOne(WaveTrack * track,
    // Take the output track and insert it in place of the original
    // sample data
 	if (bLoopSuccess) {
-		track->Clear(mT0, mT1);
-		track->Paste(mT0, outputTrack);
+
+      double mT1Dashed = mT0 + (mT1 - mT0)/(m_PercentChange/100.0 + 1.0);
+      SetTimeWarper(new LinearTimeWarper(mT0, mT0, mT1, mT1Dashed ));
+      track->ClearAndPaste(mCurT0, mCurT1, outputTrack, true, false, mOutputTracks, true, first, GetTimeWarper());
+
+      /*
+      if (first)
+      {
+         first = false;
+      }*/
 	}
 
 	double newLength = outputTrack->GetEndTime(); 
@@ -228,35 +293,26 @@ enum {
    ID_TEXT_PERCENTCHANGE = 10001,
    ID_SLIDER_PERCENTCHANGE,
    ID_CHOICE_FROMVINYL,
-   ID_CHOICE_TOVINYL,
-	ID_BUTTON_PREVIEW
+   ID_CHOICE_TOVINYL
 };
 
 
 // event table for ChangeSpeedDialog
 
-BEGIN_EVENT_TABLE(ChangeSpeedDialog, wxDialog)
-    EVT_BUTTON(wxID_OK, ChangeSpeedDialog::OnOk)
-    EVT_BUTTON(wxID_CANCEL, ChangeSpeedDialog::OnCancel)
-
+BEGIN_EVENT_TABLE(ChangeSpeedDialog, EffectDialog)
     EVT_TEXT(ID_TEXT_PERCENTCHANGE, ChangeSpeedDialog::OnText_PercentChange)
     EVT_SLIDER(ID_SLIDER_PERCENTCHANGE, ChangeSpeedDialog::OnSlider_PercentChange)
     EVT_CHOICE(ID_CHOICE_FROMVINYL, ChangeSpeedDialog::OnChoice_FromVinyl)
     EVT_CHOICE(ID_CHOICE_TOVINYL, ChangeSpeedDialog::OnChoice_ToVinyl)
 
-    EVT_BUTTON(ID_BUTTON_PREVIEW, ChangeSpeedDialog::OnPreview)
+    EVT_BUTTON(ID_EFFECT_PREVIEW, ChangeSpeedDialog::OnPreview)
 END_EVENT_TABLE()
 
-ChangeSpeedDialog::ChangeSpeedDialog(EffectChangeSpeed * effect,
-													wxWindow * parent, wxWindowID id,
-													const wxString & title, 
-													const wxPoint & position, 
-													const wxSize & size, 
-													long style)
-: wxDialog(parent, id, title, position, size, style)
+ChangeSpeedDialog::ChangeSpeedDialog(EffectChangeSpeed *effect, wxWindow *parent)
+:  EffectDialog(parent, _("Change Speed"), PROCESS_EFFECT),
+   mEffect(effect)
 {
    m_bLoopDetect = false;
-	m_pEffect = effect;
 
 	// NULL out these control members because there are some cases where the 
 	// event table handlers get called during this method, and those handlers that 
@@ -271,128 +327,71 @@ ChangeSpeedDialog::ChangeSpeedDialog(EffectChangeSpeed * effect,
 	m_FromVinyl = 0; 
 	m_ToVinyl = 0; 
 
-	
-	// CREATE THE CONTROLS PROGRAMMATICALLY.
-	wxStaticText * pStaticText;
-
-   wxBoxSizer * pBoxSizer_Dialog = new wxBoxSizer(wxVERTICAL);
-
-	// heading
-   pStaticText = new wxStaticText(this, -1, 
-												_("Change Speed, affecting both Tempo and Pitch"),
-												wxDefaultPosition, wxDefaultSize, 0);
-   pBoxSizer_Dialog->Add(pStaticText, 0, wxALIGN_CENTER | wxALL, 8);
-
-   pStaticText = new wxStaticText(this, -1, 
-												_("by Vaughan Johnson && Dominic Mazzoni"),
-												wxDefaultPosition, wxDefaultSize, 0);
-   pBoxSizer_Dialog->Add(pStaticText, 0, wxALIGN_CENTER | wxTOP | wxLEFT | wxRIGHT, 8);
-
-   pStaticText = new wxStaticText(this, -1, 
-												_("using SampleRate, by Erik de Castro Lopo"),
-												wxDefaultPosition, wxDefaultSize, 0);
-   pBoxSizer_Dialog->Add(pStaticText, 0, wxALIGN_CENTER | wxBOTTOM | wxLEFT | wxRIGHT, 8);
-
-
-	// percent change controls
-
-	// Group percent controls with spacers, 
-	// rather than static box, so they don't look isolated.
-   pBoxSizer_Dialog->Add(0, 8, 0); // spacer
-
-   wxBoxSizer * pBoxSizer_PercentChange = new wxBoxSizer(wxHORIZONTAL);
-   
-   pStaticText = new wxStaticText(this, -1, _("Percent Change:"),
-												wxDefaultPosition, wxDefaultSize, 0);
-   pBoxSizer_PercentChange->Add(pStaticText, 0, 
-											wxALIGN_CENTER_VERTICAL | wxALIGN_RIGHT | wxALL, 4);
-
-	//v Override wxTextValidator to disallow negative values <= -100.0?
-   m_pTextCtrl_PercentChange = 
-		new wxTextCtrl(this, ID_TEXT_PERCENTCHANGE, "0.0", 
-							wxDefaultPosition, wxSize(40, -1), 0,
-							wxTextValidator(wxFILTER_NUMERIC));
-   pBoxSizer_PercentChange->Add(m_pTextCtrl_PercentChange, 0, 
-											wxALIGN_CENTER_VERTICAL | wxALIGN_LEFT | wxALL, 4);
-
-   pBoxSizer_Dialog->Add(pBoxSizer_PercentChange, 0, wxALIGN_CENTER | wxALL, 4);
-
-   m_pSlider_PercentChange = 
-		new wxSlider(this, ID_SLIDER_PERCENTCHANGE, 0, 
-							PERCENTCHANGE_MIN, PERCENTCHANGE_MAX,
-							wxDefaultPosition, wxSize(100, -1), wxSL_HORIZONTAL);
-   pBoxSizer_Dialog->Add(m_pSlider_PercentChange, 1, 
-									wxGROW | wxALIGN_CENTER | wxLEFT | wxRIGHT, 4);
-
-   pBoxSizer_Dialog->Add(0, 8, 0); // spacer
-
-
-	// from/to Vinyl controls
-   wxBoxSizer * pBoxSizer_Vinyl = new wxBoxSizer(wxHORIZONTAL);
-
-	const wxString strArray_VinylRPM[] = {"33 1/3", "45", "78",
-                                         /* i18n-hint: n/a is an English
-                                            abbreviation meaning "not
-                                            applicable" (in other words,
-                                            unimportant, not relevant). */
-                                         _("n/a")};
-	const int numChoices = 4;
-
-   pStaticText = new wxStaticText(this, -1, _("Standard Vinyl RPM:   from"),
-									       wxDefaultPosition, wxDefaultSize, 0);
-   pBoxSizer_Vinyl->Add(pStaticText, 0, 
-								wxALIGN_CENTER_VERTICAL | wxALIGN_RIGHT | wxALL, 4);
-
-   m_pChoice_FromVinyl = 
-		new wxChoice(this, ID_CHOICE_FROMVINYL, 
-							wxDefaultPosition, wxSize(64, -1), numChoices, strArray_VinylRPM);
-   pBoxSizer_Vinyl->Add(m_pChoice_FromVinyl, 0, wxALIGN_CENTER | wxALL, 4);
-
-   pStaticText = new wxStaticText(this, -1, _("to"),
-									       wxDefaultPosition, wxDefaultSize, 0);
-   pBoxSizer_Vinyl->Add(pStaticText, 0, 
-								wxALIGN_CENTER_VERTICAL | wxALIGN_RIGHT | wxALL, 4);
-
-   m_pChoice_ToVinyl = 
-		new wxChoice(this, ID_CHOICE_TOVINYL, 
-							wxDefaultPosition, wxSize(64, -1), numChoices, strArray_VinylRPM);
-   pBoxSizer_Vinyl->Add(m_pChoice_ToVinyl, 0, wxALIGN_CENTER | wxALL, 4);
-
-   pBoxSizer_Dialog->Add(pBoxSizer_Vinyl, 0, wxALIGN_CENTER | wxALL, 4);
-
-
-	// Preview, OK, & Cancel buttons
-   pBoxSizer_Dialog->Add(0, 8, 0); // spacer
-
-   wxBoxSizer * pBoxSizer_OK = new wxBoxSizer(wxHORIZONTAL);
-
-   wxButton * pButton_Preview = 
-		new wxButton(this, ID_BUTTON_PREVIEW, m_pEffect->GetPreviewName());
-   pBoxSizer_OK->Add(pButton_Preview, 0, wxALIGN_CENTER | wxALL, 4);
-   pBoxSizer_OK->Add(32, 8); // horizontal spacer
-
-   wxButton * pButton_Cancel =
-       new wxButton(this, wxID_CANCEL, _("Cancel"), wxDefaultPosition, wxDefaultSize, 0);
-   pBoxSizer_OK->Add(pButton_Cancel, 0, wxALIGN_CENTER | wxALL, 4);
-
-   wxButton * pButton_OK =
-       new wxButton(this, wxID_OK, _("OK"), wxDefaultPosition, wxDefaultSize, 0);
-   pButton_OK->SetDefault();
-   pButton_OK->SetFocus();
-   pBoxSizer_OK->Add(pButton_OK, 0, wxALIGN_CENTER | wxALL, 4);
-
-   pBoxSizer_Dialog->Add(pBoxSizer_OK, 0, wxALIGN_CENTER | wxALL, 8);
-
-
-   this->SetAutoLayout(true);
-   this->SetSizer(pBoxSizer_Dialog);
-   pBoxSizer_Dialog->Fit(this);
-   pBoxSizer_Dialog->SetSizeHints(this);
+   Init();
 }
 
-bool ChangeSpeedDialog::Validate()
+void ChangeSpeedDialog::PopulateOrExchange(ShuttleGui & S)
 {
-   return true; 
+   wxTextValidator numvld(wxFILTER_NUMERIC);
+   wxArrayString rpm;
+   
+   rpm.Add(wxT("33 1/3"));
+   rpm.Add(wxT("45"));
+   rpm.Add(wxT("78"));
+  /* i18n-hint: n/a is an English abbreviation meaning "not applicable"
+     (in other words, unimportant, not relevant). */
+   rpm.Add(_("n/a"));
+
+   S.SetBorder(10);
+   S.StartHorizontalLay(wxCENTER, false);
+   {
+      S.AddTitle(_("Change Speed, affecting both Tempo and Pitch") +
+                 wxString(wxT("\n\n")) +
+                 _("by Vaughan Johnson && Dominic Mazzoni") +
+                 wxString(wxT("\n")) +
+                 _("using SampleRate, by Erik de Castro Lopo"));
+   }
+   S.EndHorizontalLay();
+   S.SetBorder(5);
+
+   //
+   S.StartMultiColumn(2, wxCENTER);
+   {
+      m_pTextCtrl_PercentChange = S.Id(ID_TEXT_PERCENTCHANGE)
+         .AddTextBox(_("Percent Change:"), wxT(""), 12);
+      m_pTextCtrl_PercentChange->SetValidator(numvld);
+   }
+   S.EndMultiColumn();
+
+   //
+   S.StartHorizontalLay(wxEXPAND);
+   {
+      S.SetStyle(wxSL_HORIZONTAL);
+      m_pSlider_PercentChange = S.Id(ID_SLIDER_PERCENTCHANGE)
+         .AddSlider(wxT(""), 0, (int)PERCENTCHANGE_MAX, (int)PERCENTCHANGE_MIN);
+      m_pSlider_PercentChange->SetName(_("Percent Change"));
+   }
+   S.EndHorizontalLay();
+
+   // 
+   S.StartMultiColumn(5, wxCENTER);
+   {
+      //
+      S.AddUnits(_("Standard Vinyl RPM:"));
+
+      m_pChoice_FromVinyl = S.Id(ID_CHOICE_FROMVINYL)
+         .AddChoice(_("from"), wxT(""), &rpm);
+      m_pChoice_FromVinyl->SetName(_("From RPM"));
+      m_pChoice_FromVinyl->SetSizeHints(100, -1);
+
+      m_pChoice_ToVinyl = S.Id(ID_CHOICE_TOVINYL)
+         .AddChoice(_("to"), wxT(""), &rpm);
+      m_pChoice_ToVinyl->SetName(_("To RPM"));
+      m_pChoice_ToVinyl->SetSizeHints(100, -1);
+   }
+   S.EndMultiColumn();
+
+   return;
 }
 
 bool ChangeSpeedDialog::TransferDataToWindow()
@@ -509,42 +508,31 @@ void ChangeSpeedDialog::OnChoice_ToVinyl(wxCommandEvent & event)
    }
 }
 
-
 void ChangeSpeedDialog::OnPreview(wxCommandEvent &event)
 {
    TransferDataFromWindow();
 
 	// Save & restore parameters around Preview, because we didn't do OK.
-	double oldPercentChange = m_pEffect->m_PercentChange;
-   m_pEffect->m_PercentChange = m_PercentChange;
-   m_pEffect->Preview();
-   m_pEffect->m_PercentChange = oldPercentChange; 
+	double oldPercentChange = mEffect->m_PercentChange;
+   if( m_PercentChange < -99.0)
+   {
+      m_PercentChange = -99.0;
+      this->Update_Text_PercentChange();
+   }
+   mEffect->m_PercentChange = m_PercentChange;
+   mEffect->Preview();
+   mEffect->m_PercentChange = oldPercentChange; 
 }
 
-void ChangeSpeedDialog::OnOk(wxCommandEvent & event)
-{
-   TransferDataFromWindow();
-   
-   if (Validate()) 
-      EndModal(true);
-   else 
-      event.Skip();
-}
-
-void ChangeSpeedDialog::OnCancel(wxCommandEvent & event)
-{
-   EndModal(false);
-}
-
-
 // helper fns
 
 void ChangeSpeedDialog::Update_Text_PercentChange()
 {
 	if (m_pTextCtrl_PercentChange) {
 		wxString str;
-		str.Printf("%.3f", m_PercentChange);
+		str.Printf(wxT("%.3f"), m_PercentChange);
 		m_pTextCtrl_PercentChange->SetValue(str);
+      FindWindow(wxID_OK)->Enable(m_PercentChange > -100.0);
 	}
 }
 
@@ -564,13 +552,14 @@ void ChangeSpeedDialog::Update_Slider_PercentChange()
 void ChangeSpeedDialog::Update_Vinyl() 
 // Update Vinyl controls for new percent change.
 {
-	if (m_pChoice_ToVinyl) 
+	if (m_pChoice_ToVinyl) {
 		// Chances are so low that the slider will exactly match a 
 		// standard ratio, just turn it "n/a" unless it's 0.0.
 		if ((m_PercentChange == 0.0) && m_pChoice_FromVinyl)
 			m_pChoice_ToVinyl->SetSelection(m_pChoice_FromVinyl->GetSelection());
 		else
 			m_pChoice_ToVinyl->SetSelection(CHOICE_NA);
+        }
 }
 
 void ChangeSpeedDialog::Update_PercentChange() 
@@ -598,3 +587,15 @@ void ChangeSpeedDialog::Update_PercentChange()
 		this->Update_Slider_PercentChange();
 	}
 }
+
+// Indentation settings for Vim and Emacs and unique identifier for Arch, a
+// version control system. Please do not modify past this point.
+//
+// Local Variables:
+// c-basic-offset: 3
+// indent-tabs-mode: nil
+// End:
+//
+// vim: et sts=3 sw=3
+// arch-tag: 84775e31-325f-46fd-bcec-7336b0a574e1
+
