@@ -5,16 +5,41 @@
   Meter.cpp
 
   Dominic Mazzoni
+  Vaughan Johnson
 
-  VU Meter, for displaying recording/playback level
+  2004.06.25 refresh rate limited to 30mS, by ChackoN
+
+*******************************************************************//**
+
+\class Meter
+\brief VU Meter, for displaying recording/playback level
 
   This is a bunch of common code that can display many different
   forms of VU meters and other displays.
 
-**********************************************************************/
+  But note that a lot of later code here assumes these are 
+  MeterToolBar meters, e.g., Meter::StartMonitoring, 
+  so these are not as generic/common as originally intended.
+
+*//****************************************************************//**
+
+\class MeterBar
+\brief A struct used by Meter to hold the position of one bar.
+
+*//****************************************************************//**
+
+\class MeterUpdateMsg
+\brief Message used to update the Meter
+
+*//****************************************************************//**
+
+\class MeterUpdateQueue
+\brief Queue of MeterUpdateMsg used to feed the Meter.
+
+*//******************************************************************/
 
 #include "../Audacity.h"
-#include "../Prefs.h" // for gPrefs
+#include "../AudacityApp.h"
 
 #include <wx/defs.h>
 #include <wx/dcmemory.h>
@@ -22,7 +47,14 @@
 #include <wx/intl.h>
 #include <wx/menu.h>
 #include <wx/settings.h>
+#include <wx/textdlg.h>
+#include <wx/numdlg.h>
 #include <wx/tooltip.h>
+#include <wx/msgdlg.h>
+
+#if defined(__WXMAC__)
+#include <wx/mac/uma.h>
+#endif
 
 #include <math.h>
 
@@ -31,10 +63,45 @@
 #include "../AudioIO.h"
 #include "../AColor.h"
 #include "../ImageManipulation.h"
-#include "../../images/MixerImages.h"
+//#include "../../images/MixerImages.h"
 #include "../Project.h"
-#include "../MeterToolBar.h"
-#include "Warning.h"
+#include "../toolbars/MeterToolBar.h"
+#include "../toolbars/ControlToolBar.h"
+#include "../Prefs.h"
+
+#include "../Theme.h"
+#include "../AllThemeResources.h"
+#include "../Experimental.h"
+
+
+/* Updates to the meter are passed accross via meter updates, each contained in
+ * a MeterUpdateMsg object */
+wxString MeterUpdateMsg::toString()
+{
+wxString output;  // somewhere to build up a string in
+output = wxString::Format(wxT("Meter update msg: %i channels, %i samples\n"), \
+      kMaxMeterBars, numFrames);
+for (int i = 0; i<kMaxMeterBars; i++)
+   {  // for each channel of the meters
+   output += wxString::Format(wxT("%f peak, %f rms "), peak[i], rms[i]);
+   if (clipping[i])
+      output += wxString::Format(wxT("clipped "));
+   else
+      output += wxString::Format(wxT("no clip "));
+   output += wxString::Format(wxT("%i head, %i tail\n"), headPeakCount[i], tailPeakCount[i]);
+   }
+return output;
+}
+
+wxString MeterUpdateMsg::toStringIfClipped()
+{
+for (int i = 0; i<kMaxMeterBars; i++)
+   {
+   if (clipping[i] || (headPeakCount[i] > 0) || (tailPeakCount[i] > 0))
+      return toString();
+   }
+return wxT("");
+}
 
 //
 // The Meter passes itself messages via this queue so that it can
@@ -73,6 +140,8 @@ bool MeterUpdateQueue::Put(MeterUpdateMsg &msg)
    if (len >= mBufferSize-1)
       return false;
 
+   //wxLogDebug(wxT("Put: %s"), msg.toString().c_str());
+
    mBuffer[mEnd] = msg;
    mEnd = (mEnd+1)%mBufferSize;
 
@@ -100,7 +169,10 @@ bool MeterUpdateQueue::Get(MeterUpdateMsg &msg)
 
 enum {
    OnMeterUpdateID = 6000,
+   OnDisableMeterID,
+   OnMonitorID,
    OnHorizontalID,
+   OnAutomatedInputLevelAdjustmentID,
    OnVerticalID,
    OnMultiID,
    OnEqualizerID,
@@ -108,16 +180,17 @@ enum {
    OnLinearID,
    OnDBID,
    OnClipID,
-   OnMonitorID,
    OnFloatID,
-   OnDisableID
+   OnPreferencesID
 };
 
 BEGIN_EVENT_TABLE(Meter, wxPanel)
    EVT_TIMER(OnMeterUpdateID, Meter::OnMeterUpdate)
    EVT_MOUSE_EVENTS(Meter::OnMouse)
+   EVT_ERASE_BACKGROUND(Meter::OnErase)
    EVT_PAINT(Meter::OnPaint)
    EVT_SIZE(Meter::OnSize)
+   EVT_MENU(OnDisableMeterID, Meter::OnDisableMeter)
    EVT_MENU(OnHorizontalID, Meter::OnHorizontal)
    EVT_MENU(OnVerticalID, Meter::OnVertical)
    EVT_MENU(OnMultiID, Meter::OnMulti)
@@ -127,8 +200,11 @@ BEGIN_EVENT_TABLE(Meter, wxPanel)
    EVT_MENU(OnDBID, Meter::OnDB)
    EVT_MENU(OnClipID, Meter::OnClip)
    EVT_MENU(OnMonitorID, Meter::OnMonitor)
+#ifdef AUTOMATED_INPUT_LEVEL_ADJUSTMENT
+   EVT_MENU(OnAutomatedInputLevelAdjustmentID, Meter::OnAutomatedInputLevelAdjustment)
+#endif
    EVT_MENU(OnFloatID, Meter::OnFloat)
-   EVT_MENU(OnDisableID, Meter::OnDisable)
+   EVT_MENU(OnPreferencesID, Meter::OnPreferences)
 END_EVENT_TABLE()
 
 IMPLEMENT_CLASS(Meter, wxPanel)
@@ -136,16 +212,18 @@ IMPLEMENT_CLASS(Meter, wxPanel)
 Meter::Meter(wxWindow* parent, wxWindowID id,
              bool isInput,
              const wxPoint& pos /*= wxDefaultPosition*/,
-             const wxSize& size /*= wxDefaultSize*/):
-   wxPanel(parent, id, pos, size),
-   mQueue(256),
+             const wxSize& size /*= wxDefaultSize*/, 
+             Style style /*= HorizontalStereo*/, 
+             float fDecayRate /*= 60.0f*/)
+: wxPanel(parent, id, pos, size),
+   mQueue(1024),
    mWidth(size.x), mHeight(size.y),
    mIsInput(isInput),
-   mStyle(HorizontalStereo),
+   mStyle(style),
    mDB(true),
-   mDBRange(60),
+   mDBRange(ENV_DB_RANGE),
    mDecay(true),
-   mDecayRate(60),
+   mDecayRate(fDecayRate),
    mClip(true),
    mNumPeakSamplesToClip(3),
    mPeakHoldDuration(3),
@@ -154,12 +232,18 @@ Meter::Meter(wxWindow* parent, wxWindowID id,
    mNumBars(0),
    mLayoutValid(false),
    mBitmap(NULL),
-   mBackgroundBitmap(NULL),
    mIcon(NULL)
 {
    int i;
-   
-   wxColour origColour(204, 204, 204);
+
+    wxColour backgroundColour =
+      wxSystemSettings::GetColour(wxSYS_COLOUR_3DFACE);
+    mBkgndBrush = wxBrush(backgroundColour, wxSOLID);
+
+   UpdatePrefs();
+
+   mPeakPeakPen = wxPen(theTheme.Colour( clrMeterPeak),        1, wxSOLID);
+   mDisabledPen = wxPen(theTheme.Colour( clrMeterDisabledPen), 1, wxSOLID);
 
    /* i18n-hint: One-letter abbreviation for Left, in VU Meter */
    mLeftText = _("L");
@@ -170,72 +254,102 @@ Meter::Meter(wxWindow* parent, wxWindowID id,
    mRightSize = wxSize(0, 0);
 
    if (mIsInput) {
-      mPen = wxPen(wxColour(204, 70, 70), 1, wxSOLID);
-      mBrush = wxBrush(wxColour(204, 70, 70), wxSOLID);
-      mRMSBrush = wxBrush(wxColour(255, 102, 102), wxSOLID);
-      mClipBrush = wxBrush(wxColour(255, 53, 53), wxSOLID);
-      mLightPen = wxPen(wxColour(255, 153, 153), 1, wxSOLID);
-      mDarkPen = wxPen(wxColour(153, 61, 61), 1, wxSOLID);
-   }
-   else {
-      mPen = wxPen(wxColour(70, 204, 70), 1, wxSOLID);
-      mBrush = wxBrush(wxColour(70, 204, 70), wxSOLID);
-      mRMSBrush = wxBrush(wxColour(102, 255, 102), wxSOLID);
-      mClipBrush = wxBrush(wxColour(255, 53, 53), wxSOLID);
-      mLightPen = wxPen(wxColour(153, 255, 153), 1, wxSOLID);
-      mDarkPen = wxPen(wxColour(61, 164, 61), 1, wxSOLID);
+      mPen       = wxPen(   theTheme.Colour( clrMeterInputPen         ), 1, wxSOLID);
+      mBrush     = wxBrush( theTheme.Colour( clrMeterInputBrush       ), wxSOLID);
+      mRMSBrush  = wxBrush( theTheme.Colour( clrMeterInputRMSBrush    ), wxSOLID);
+      mClipBrush = wxBrush( theTheme.Colour( clrMeterInputClipBrush   ), wxSOLID);
+      mLightPen  = wxPen(   theTheme.Colour( clrMeterInputLightPen    ), 1, wxSOLID);
+      mDarkPen   = wxPen(   theTheme.Colour( clrMeterInputDarkPen     ), 1, wxSOLID);
+   }   
+   else {                                                         
+      mPen       = wxPen(   theTheme.Colour( clrMeterOutputPen        ), 1, wxSOLID);
+      mBrush     = wxBrush( theTheme.Colour( clrMeterOutputBrush      ), wxSOLID);
+      mRMSBrush  = wxBrush( theTheme.Colour( clrMeterOutputRMSBrush   ), wxSOLID);
+      mClipBrush = wxBrush( theTheme.Colour( clrMeterOutputClipBrush  ), wxSOLID);
+      mLightPen  = wxPen(   theTheme.Colour( clrMeterOutputLightPen   ), 1, wxSOLID);
+      mDarkPen   = wxPen(   theTheme.Colour( clrMeterOutputDarkPen    ), 1, wxSOLID);
    }
 
-   CreateIcon(2);
+   mDisabledBkgndBrush = wxBrush(theTheme.Colour( clrMeterDisabledBrush), wxSOLID);
+//   mDisabledBkgndBrush = wxBrush(
+//            wxSystemSettings::GetSystemColour(wxSYS_COLOUR_3DSHADOW), wxSOLID);
+//            wxSystemSettings::GetSystemColour(wxSYS_COLOUR_3DLIGHT), wxSOLID);
+   if (mMeterDisabled) {
+      mSavedBkgndBrush = mBkgndBrush;
+      mSavedBrush = mBrush;
+      mSavedRMSBrush = mRMSBrush;
 
-   // The new Ruler is much smarter at picking fonts than we are...
-   //mRuler.SetFonts(GetFont(), GetFont());
+      mBkgndBrush = mDisabledBkgndBrush;
+      mBrush = mDisabledBkgndBrush;
+      mRMSBrush = mDisabledBkgndBrush;
+   }
+
+   // MixerTrackCluster style has no menu, so disallows SetStyle, so never needs icon.
+   if (mStyle != MixerTrackCluster)
+      CreateIcon(2);
+
+   mRuler.SetFonts(GetFont(), GetFont(), GetFont());
 
    mTimer.SetOwner(this, OnMeterUpdateID);
    Reset(44100.0, true);
-   for(i=0; i<kMaxMeterBars; i++)
+   for(i=0; i<kMaxMeterBars; i++) {
       mBar[i].clipping = false;
+      mBar[i].isclipping = false;
+   }
+}
+
+void Meter::Clear()
+{
+   mQueue.Clear();
 }
 
 void Meter::CreateIcon(int aquaOffset)
 {
-   wxColour backgroundColour =
-      wxSystemSettings::GetSystemColour(wxSYS_COLOUR_3DFACE);
-   mBkgndBrush = wxBrush(backgroundColour, wxSOLID);
-   wxImage *image, *alpha;
-
+   /// \todo Remove wasteful delete/new pair.  It is done in every call to layout.
    if (mIcon) {
       delete mIcon;
       mIcon = NULL;
    }
-
-   if (mIsInput) {
-      image = new wxImage(wxBitmap(Mic).ConvertToImage());
-      alpha = new wxImage(wxBitmap(MicAlpha).ConvertToImage());
+   if(mIsInput)
+   {
+      /// JKC: !!!! If you pass theTheme.Bitmap(bmpMic) you get a white rather than a black mic.
+      /// Weird behaviour in wxWidgets, I guess.
+      mIcon = new wxBitmap(theTheme.Image( bmpMic ));
    }
-   else {
-      image = new wxImage(wxBitmap(Speaker).ConvertToImage());
-      alpha = new wxImage(wxBitmap(SpeakerAlpha).ConvertToImage());
+   else
+   {
+      mIcon = new wxBitmap(theTheme.Image( bmpSpeaker ));
    }
-
-   wxImage *bkgnd = CreateSysBackground(25, 25, aquaOffset,
-					backgroundColour);
-   wxImage *final = OverlayImage(bkgnd, image, alpha, 0, 0);
-   mIcon = new wxBitmap(final);
-
-   delete image;
-   delete alpha;
-   delete bkgnd;
-   delete final;
 }
 
 Meter::~Meter()
 {
+   // LLL:  This prevents a crash during termination if monitoring
+   //       is active.
+   if (gAudioIO->IsMonitoring())
+      gAudioIO->StopStream();
    delete mIcon;
    if (mBitmap)
       delete mBitmap;
-   if (mBackgroundBitmap)
-      delete mBackgroundBitmap;
+}
+
+void Meter::UpdatePrefs()
+{
+   mDBRange = gPrefs->Read(wxT("/GUI/EnvdBRange"), ENV_DB_RANGE);
+   mMeterRefreshRate = gPrefs->Read(wxT("/Meter/MeterRefreshRate"), 30);
+
+   // MixerTrackCluster style has no menu, so disallows disabling the meter.
+   if (mStyle == MixerTrackCluster)
+      mMeterDisabled = 0L;
+   else if (mIsInput) 
+      mMeterDisabled = gPrefs->Read(wxT("/Meter/MeterInputDisabled"), (long)0);
+   else 
+      mMeterDisabled = gPrefs->Read(wxT("/Meter/MeterOutputDisabled"), (long)0);
+}
+
+void Meter::OnErase(wxEraseEvent &evt)
+{
+   // Ignore it to prevent flashing
 }
 
 void Meter::OnPaint(wxPaintEvent &evt)
@@ -260,57 +374,87 @@ void Meter::OnSize(wxSizeEvent &evt)
    delete mBitmap;
    mBitmap = NULL;
    GetClientSize(&mWidth, &mHeight);
+
+//::wxMessageBox(wxString::Format(" mHeight=%d, mWidth=%d", mHeight,mWidth));
    mLayoutValid = false;
 }
 
 void Meter::OnMouse(wxMouseEvent &evt)
 {
+   if (mStyle == MixerTrackCluster) // MixerTrackCluster style has no menu.
+      return;
+
   #if wxUSE_TOOLTIPS // Not available in wxX11
    if (evt.Leaving()){
-      GetActiveProject()->TP_DisplayStatusMessage("",0);
+      GetActiveProject()->TP_DisplayStatusMessage(wxT(""));
    }
    else if (evt.Entering()) {
       // Display the tooltip in the status bar
       wxToolTip * pTip = this->GetToolTip();
       if( pTip ) {
          wxString tipText = pTip->GetTip();
-         GetActiveProject()->TP_DisplayStatusMessage(tipText, 0);
+         GetActiveProject()->TP_DisplayStatusMessage(tipText);
       }
    }
   #endif
 
    if (evt.RightDown() ||
-       (evt.ButtonDown() && mMenuRect.Inside(evt.m_x, evt.m_y))) {
+       (evt.ButtonDown() && mMenuRect.Contains(evt.m_x, evt.m_y))) 
+   {
       wxMenu *menu = new wxMenu();
       // Note: these should be kept in the same order as the enum
+      if (mMeterDisabled)
+         menu->Append(OnDisableMeterID, _("Enable Meter"));
+      else
+         menu->Append(OnDisableMeterID, _("Disable Meter"));
+      if (mIsInput) {
+         if (gAudioIO->IsMonitoring())
+            menu->Append(OnMonitorID, _("Stop Monitoring"));
+         else
+            menu->Append(OnMonitorID, _("Start Monitoring"));
+
+         #ifdef AUTOMATED_INPUT_LEVEL_ADJUSTMENT
+            if (gAudioIO->AILAIsActive())
+               menu->Append(OnAutomatedInputLevelAdjustmentID, _("Stop Automated Input Level Adjustment"));
+            else
+               menu->Append(OnAutomatedInputLevelAdjustmentID, _("Start Automated Input Level Adjustment"));
+
+            bool AVActive;
+            gPrefs->Read(wxT("/AudioIO/AutomatedInputLevelAdjustment"), &AVActive, false);
+            if (!AVActive || !GetActiveProject()->GetControlToolBar()->IsRecordDown())
+               menu->Enable(OnAutomatedInputLevelAdjustmentID, false);
+         #endif
+
+      }
+      menu->AppendSeparator();
+
       menu->Append(OnHorizontalID, _("Horizontal Stereo"));
       menu->Append(OnVerticalID, _("Vertical Stereo"));
       //menu->Append(OnMultiID, _("Vertical Multichannel"));
       //menu->Append(OnEqualizerID, _("Equalizer"));
       //menu->Append(OnWaveformID, _("Waveform"));
-      menu->Enable(OnHorizontalID + mStyle, false);
+      //menu->Enable(OnHorizontalID + mStyle, false);
+      menu->Enable(mStyle==VerticalStereo? OnVerticalID: OnHorizontalID, false);
       menu->AppendSeparator();
+
       menu->Append(OnLinearID, _("Linear"));
       menu->Append(OnDBID, _("dB"));
       menu->Enable(mDB? OnDBID: OnLinearID, false);
       //menu->AppendSeparator();
       //menu->Append(OnClipID, _("Turn on clipping"));
-      if (mIsInput) {
-         menu->AppendSeparator();
-         menu->Append(OnMonitorID, _("Monitor input"));
-      }
       //menu->AppendSeparator();
       //menu->Append(OnFloatID, _("Float Window"));
 
       menu->AppendSeparator();
-      menu->Append(OnDisableID, _("Disable"));
+      menu->Append(OnPreferencesID, _("Preferences..."));
+
 
       if (evt.RightDown())
          PopupMenu(menu, evt.m_x, evt.m_y);
       else
          PopupMenu(menu, mMenuRect.x + 1, mMenuRect.y + mMenuRect.height + 1);
       delete menu;
-   }       
+   }
    else if (evt.ButtonDown()) {
       if (mIsInput)
          StartMonitoring();
@@ -319,6 +463,9 @@ void Meter::OnMouse(wxMouseEvent &evt)
 
 void Meter::SetStyle(Meter::Style newStyle)
 {
+   // MixerTrackCluster disallows style change.
+   if (mStyle == MixerTrackCluster)
+      return;
    mStyle = newStyle;
    mLayoutValid = false;
    Refresh(true);
@@ -330,7 +477,7 @@ void Meter::Reset(double sampleRate, bool resetClipping)
 
    mT = 0;
    mRate = sampleRate;
-   for(j=0; j<mNumBars; j++)
+   for(j=0; j<kMaxMeterBars; j++)
       ResetBar(&mBar[j], resetClipping);
 
    // wxTimers seem to be a little unreliable - sometimes they stop for
@@ -342,7 +489,7 @@ void Meter::Reset(double sampleRate, bool resetClipping)
    while(mQueue.Get(msg)) {
    }
 
-   mTimer.Start(25); // every 25 ms -> ~40 updates per second
+   mTimer.Start(1000 / mMeterRefreshRate);
 
    mLayoutValid = false;
    Refresh(false);
@@ -406,11 +553,9 @@ void Meter::UpdateDisplay(int numChannels, int numFrames, float *sampleData)
 
          // In addition to looking for mNumPeakSamplesToClip peaked
          // samples in a row, also send the number of peaked samples
-         // at the head and tail, in case there's a run of 
-         // Send the number of peaked samples at the head and tail,
-         // in case there's a run of peaked samples that crosses
-         // block boundaries
-         if (fabs(sptr[j])>=1.0) {
+         // at the head and tail, in case there's a run of peaked samples
+         // that crosses block boundaries
+         if (fabs(sptr[j])>=MAX_AUDIO) {
             if (msg.headPeakCount[j]==i)
                msg.headPeakCount[j]++;
             msg.tailPeakCount[j]++;
@@ -435,11 +580,63 @@ void Meter::UpdateDisplay(int numChannels, int numFrames, float *sampleData)
    mQueue.Put(msg);
 }
 
+void Meter::UpdateDisplay(int numChannels, int numFrames, 
+                           // Need to make these double-indexed arrays if we handle more than 2 channels.
+                           float* maxLeft, float* rmsLeft, 
+                           float* maxRight, float* rmsRight, 
+                           const sampleCount kSampleCount)
+{
+   int i, j;
+   int num = intmin(numChannels, mNumBars);
+   MeterUpdateMsg msg;
+
+   msg.numFrames = kSampleCount;
+   for(j=0; j<mNumBars; j++) {
+      msg.peak[j] = 0.0;
+      msg.rms[j] = 0.0;
+      msg.clipping[j] = false;
+      msg.headPeakCount[j] = 0;
+      msg.tailPeakCount[j] = 0;
+   }
+
+   for(i=0; i<numFrames; i++) {
+      for(j=0; j<num; j++) {
+         msg.peak[j] = floatMax(msg.peak[j], ((j == 0) ? maxLeft[i] : maxRight[i]));
+         msg.rms[j] = floatMax(msg.rms[j], ((j == 0) ? rmsLeft[i] : rmsRight[i]));
+
+         // In addition to looking for mNumPeakSamplesToClip peaked
+         // samples in a row, also send the number of peaked samples
+         // at the head and tail, in case there's a run 
+         // of peaked samples that crosses block boundaries.
+         if (fabs((j == 0) ? maxLeft[i] : maxRight[i]) >= MAX_AUDIO)
+         {
+            if (msg.headPeakCount[j]==i)
+               msg.headPeakCount[j]++;
+            msg.tailPeakCount[j]++;
+            if (msg.tailPeakCount[j] > mNumPeakSamplesToClip)
+               msg.clipping[j] = true;
+         }
+         else
+            msg.tailPeakCount[j] = 0;
+      }
+   }
+
+   if (mDB) {
+      for(j=0; j<mNumBars; j++) {
+         msg.peak[j] = ToDB(msg.peak[j], mDBRange);
+         msg.rms[j] = ToDB(msg.rms[j], mDBRange);
+      }
+   }
+
+   mQueue.Put(msg);
+}
+
 void Meter::OnMeterUpdate(wxTimerEvent &evt)
 {
    MeterUpdateMsg msg;
    int numChanges = 0;
-
+   double maxPeak = 0.0;
+   bool discarded = false;
    // There may have been several update messages since the last
    // time we got to this function.  Catch up to real-time by
    // popping them off until there are none left.  It is necessary
@@ -450,8 +647,16 @@ void Meter::OnMeterUpdate(wxTimerEvent &evt)
       double deltaT = msg.numFrames / mRate;
       int j;
       
+      // <Why is this in the loop, rather than top of the method? 
+      //    Or just a condition on the following, so we pop all the msgs while disabled?>
+      if (mMeterDisabled)
+         return;
+      //wxLogDebug(wxT("Pop: %s"), msg.toString().c_str());
+
       mT += deltaT;
       for(j=0; j<mNumBars; j++) {
+         mBar[j].isclipping = false;
+
          if (mDecay) {
             if (mDB) {
                float decayAmount = mDecayRate * deltaT / mDBRange;
@@ -467,9 +672,10 @@ void Meter::OnMeterUpdate(wxTimerEvent &evt)
          }
          else
             mBar[j].peak = msg.peak[j];
-         
+
          // This smooths out the RMS signal
-         mBar[j].rms = mBar[j].rms * 0.9 + msg.rms[j] * 0.1;
+         float smooth = pow(0.9, (double)msg.numFrames/1024.0);
+         mBar[j].rms = mBar[j].rms * smooth + msg.rms[j] * (1.0 - smooth);
          
          if (mT - mBar[j].peakHoldTime > mPeakHoldDuration ||
              mBar[j].peak > mBar[j].peakHold) {
@@ -477,16 +683,58 @@ void Meter::OnMeterUpdate(wxTimerEvent &evt)
             mBar[j].peakHoldTime = mT;
          }
          
+         if (mBar[j].peak > mBar[j].peakPeakHold )
+            mBar[j].peakPeakHold = mBar[j].peak;
+         
          if (msg.clipping[j] ||
              mBar[j].tailPeakCount+msg.headPeakCount[j] >=
-             mNumPeakSamplesToClip)
+             mNumPeakSamplesToClip){
             mBar[j].clipping = true;
+            mBar[j].isclipping = true;
+         }
+
          mBar[j].tailPeakCount = msg.tailPeakCount[j];
-      }      
+#ifdef AUTOMATED_INPUT_LEVEL_ADJUSTMENT
+         if (mT > gAudioIO->AILAGetLastDecisionTime()) {
+            discarded = false;
+            maxPeak = msg.peak[j] > maxPeak ? msg.peak[j] : maxPeak;
+            printf("%f@%f ", msg.peak[j], mT);
+         }
+         else {
+	         discarded = true;
+            printf("%f@%f discarded\n", msg.peak[j], mT);
+	      }
+#endif
+      }
    } // while
 
-   if (numChanges > 0)      
+   if (numChanges > 0) {
+      #ifdef AUTOMATED_INPUT_LEVEL_ADJUSTMENT
+         if (gAudioIO->AILAIsActive() && mIsInput && !discarded) {
+            gAudioIO->AILAProcess(maxPeak);
+            putchar('\n');
+         }
+      #endif
       RepaintBarsNow();
+   }
+}
+
+float Meter::GetMaxPeak()
+{
+   int j;
+   float maxPeak = 0.;
+
+   for(j=0; j<mNumBars; j++)
+      maxPeak = mBar[j].peak > maxPeak ? mBar[j].peak : maxPeak;
+
+   return(maxPeak);
+}
+
+double Meter::ToLinearIfDB(double value)
+{
+   if (mDB)
+      value = pow(10.0, (-(1.0-value)*mDBRange)/20.0);
+   return value;
 }
 
 wxFont Meter::GetFont()
@@ -506,16 +754,35 @@ void Meter::ResetBar(MeterBar *b, bool resetClipping)
    b->peakHold = 0.0;
    b->peakHoldTime = 0.0;
    if (resetClipping)
+   {
       b->clipping = false;
+      b->peakPeakHold =0.0;
+   }
+   b->isclipping = false;
    b->tailPeakCount = 0;
 }
 
+bool Meter::IsClipping()
+{
+   for (int c = 0; c < kMaxMeterBars; c++)
+      if (mBar[c].isclipping)
+         return true;
+   return false;
+}
+
 void Meter::HandleLayout()
 {
-   int iconWidth = mIcon->GetWidth();
-   int iconHeight = mIcon->GetHeight();
-   int menuWidth = 17;
-   int menuHeight = 11;
+   int iconWidth = 0;
+   int iconHeight = 0;
+   int menuWidth = 0;
+   int menuHeight = 0;
+   if (mStyle != MixerTrackCluster)
+   {
+      iconWidth = mIcon->GetWidth();
+      iconHeight = mIcon->GetHeight();
+      menuWidth = 17;
+      menuHeight = 11;
+   }
    int width = mWidth;
    int height = mHeight;
    int left = 0, top = 0;
@@ -523,25 +790,13 @@ void Meter::HandleLayout()
    int barw, barh;
    int i;
 
-#ifdef __WXMAC__
-   if (!mBackgroundBitmap ||
-       mBackgroundBitmap->GetWidth() != mWidth ||
-       mBackgroundBitmap->GetWidth() != mHeight) {
-     if (mBackgroundBitmap)
-       delete mBackgroundBitmap;
-
-     wxImage *image = CreateAquaBackground(mWidth, mHeight, 0);
-     mBackgroundBitmap = new wxBitmap(image);
-     delete image;
-   }
-#endif
-
    mRuler.SetFlip(true);
    mRuler.SetLabelEdges(true);
 
    switch(mStyle) {
    default:
-      printf("Style not handled yet!\n");
+      wxPrintf(wxT("Style not handled yet!\n"));
+      break;
    case VerticalStereo:
       mMenuRect = wxRect(mWidth - menuWidth - 5, mHeight - menuHeight - 2,
                          menuWidth, menuHeight);
@@ -550,11 +805,18 @@ void Meter::HandleLayout()
       else
          mIconPos = wxPoint(mWidth - iconWidth - 1, 1);
       width = intmin(mWidth-(iconWidth+2), mWidth-(menuWidth+3));
+   case MixerTrackCluster:
+      // Doesn't show menu, icon, or L/R labels,
+      // but otherwise like VerticalStereo.
+
       if (width >= mLeftSize.x + mRightSize.x + 24) {
-         mLeftTextPos = wxPoint(2, height-2-mLeftSize.y);
+         if (mStyle != MixerTrackCluster)
+         {
+            mLeftTextPos = wxPoint(2, height-2-mLeftSize.y);
+            left += mLeftSize.x+4;
+         }
          mRightTextPos = wxPoint(width-mLeftSize.x, height-2-mLeftSize.y);
-         left += mLeftSize.x+4;
-         width -= mLeftSize.x + mRightSize.x + 8;
+         width -= mLeftSize.x + mRightSize.x + 8; //vvvvv ...but then -8 in UmixIt? -- for vertical only?
       }
       barw = (width-2)/2;
       barh = height - 4;
@@ -581,7 +843,7 @@ void Meter::HandleLayout()
       mRuler.SetBounds(mBar[1].r.x + mBar[1].r.width + 1,
                        mBar[1].r.y,
                        mBar[1].r.x + width,
-                       mBar[1].r.height);
+                       mBar[1].r.y + mBar[1].r.height);
       if (mDB) {
          mRuler.SetRange(0, -mDBRange);
          mRuler.SetFormat(Ruler::LinearDBFormat);
@@ -590,7 +852,8 @@ void Meter::HandleLayout()
          mRuler.SetRange(1, 0);
          mRuler.SetFormat(Ruler::RealFormat);
       }
-      mRuler.OfflimitsPixels(mMenuRect.y-mBar[1].r.y, mBar[1].r.height);
+      if (mStyle != MixerTrackCluster) // MixerTrackCluster style has no menu.
+         mRuler.OfflimitsPixels(mMenuRect.y-mBar[1].r.y, mBar[1].r.height);
       break;
    case HorizontalStereo:
       if (mWidth < menuWidth + iconWidth + 8) {
@@ -600,7 +863,7 @@ void Meter::HandleLayout()
       }         
       else {
          mIconPos = wxPoint(2, mHeight - iconHeight);
-         mMenuRect = wxRect(iconWidth + 2, mHeight - menuHeight - 3,
+         mMenuRect = wxRect(iconWidth + 2, mHeight - menuHeight - 5,
                             menuWidth, menuHeight);
       }
       height = intmin(height-(menuHeight+3), height-iconHeight) - 2;
@@ -670,7 +933,9 @@ void Meter::HandleLayout()
       mAllBarsRect = wxRect(left, top, right-left+1, bottom-top+1);
    }
 
-   CreateIcon(mIconPos.y % 4);
+   // MixerTrackCluster style has no popup, so disallows SetStyle, so never needs icon.
+   if (mStyle != MixerTrackCluster) 
+      CreateIcon(mIconPos.y % 4);
 
    mLayoutValid = true;
 }
@@ -680,42 +945,68 @@ void Meter::HandlePaint(wxDC &dc)
    int i;
 
    dc.SetFont(GetFont());
-   if (mLeftSize.x == 0) {
-      dc.GetTextExtent(mLeftText, &mLeftSize.x, &mLeftSize.y);
-      dc.GetTextExtent(mRightText, &mRightSize.x, &mRightSize.y);
+   if (mLeftSize.x == 0) { // Not yet initialized to dc.
+      if (mStyle != MixerTrackCluster) // MixerTrackCluster style has no L/R labels.
+      {
+         dc.GetTextExtent(mLeftText, &mLeftSize.x, &mLeftSize.y);
+         dc.GetTextExtent(mRightText, &mRightSize.x, &mRightSize.y);
+      }
+      if ((mStyle == VerticalStereo) || (mStyle == MixerTrackCluster)) // VerticalMulti?
+      {
+         // For any vertical style, also need mRightSize big enough for Ruler width.
+         int rulerWidth;
+         int rulerHeight;
+         dc.GetTextExtent(wxT("-888"), &rulerWidth, &rulerHeight); // -888 is nice and wide.
+         if (mRightSize.x < rulerWidth)
+            mRightSize.x = rulerWidth;
+      }
    }
 
    if (!mLayoutValid)
       HandleLayout();
 
-#ifdef __WXMAC__
-   dc.DrawBitmap(*mBackgroundBitmap, 0, 0);
-#else
+#ifndef USE_AQUA_THEME
+#ifdef EXPERIMENTAL_THEMING
+   if( !mMeterDisabled )
+   {
+      mBkgndBrush.SetColour( GetParent()->GetBackgroundColour() );
+   }
+#endif
+
    dc.SetPen(*wxTRANSPARENT_PEN);
    dc.SetBrush(mBkgndBrush);
    dc.DrawRectangle(0, 0, mWidth, mHeight);
 #endif
 
-   dc.DrawBitmap(*mIcon, mIconPos.x, mIconPos.y);
+   // MixerTrackCluster style has no icon or menu.
+   if (mStyle != MixerTrackCluster)
+   {
+      dc.DrawBitmap(*mIcon, mIconPos.x, mIconPos.y, true);
 
-   dc.SetPen(*wxBLACK_PEN);
-   dc.SetBrush(*wxTRANSPARENT_BRUSH);
-   wxRect r = mMenuRect;
-   dc.DrawRectangle(r);
-   for(i=2; i<r.height-2; i++)   // Menu triangle
-      dc.DrawLine(r.x + i, r.y + i,
-                  r.x + r.width - i, r.y + i);
-   dc.DrawLine(r.x + r.width, r.y + 1,
-               r.x + r.width, r.y + r.height);
-   dc.DrawLine(r.x + 1, r.y + r.height,
-               r.x + r.width, r.y + r.height);
+      // Draws a beveled button and a down pointing triangle.
+      // The style and sizing matches the ones in the title 
+      // bar of the waveform left-hand-side panels.
+
+      wxRect r = mMenuRect;
+      AColor::Bevel(dc, true, r);
+      dc.SetBrush(*wxBLACK_BRUSH);
+      dc.SetPen(*wxBLACK_PEN);
+      AColor::Arrow(dc,
+                    r.x + 3,
+                    r.y + 5,
+                    10);
+   }
 
    if (mNumBars>0)
       mRuler.Draw(dc);
    
-   dc.SetFont(GetFont());
-   dc.DrawText(mLeftText, mLeftTextPos.x, mLeftTextPos.y);
-   dc.DrawText(mRightText, mRightTextPos.x, mRightTextPos.y);
+   // MixerTrackCluster style has no L/R labels.
+   if (mStyle != MixerTrackCluster)
+   {
+      dc.SetFont(GetFont());
+      dc.DrawText(mLeftText, mLeftTextPos.x, mLeftTextPos.y);
+      dc.DrawText(mRightText, mRightTextPos.x, mRightTextPos.y);
+   }
 
    for(i=0; i<mNumBars; i++)
       DrawMeterBar(dc, &mBar[i]);
@@ -751,7 +1042,7 @@ void Meter::RepaintBarsNow()
 void Meter::DrawMeterBar(wxDC &dc, MeterBar *meterBar)
 {
    wxRect r = meterBar->r;
-   wxRect rRMS;
+   wxRect rRMS = meterBar->r;
    dc.SetPen(*wxTRANSPARENT_PEN);
    dc.SetBrush(mBkgndBrush);
    dc.DrawRectangle(r);
@@ -761,9 +1052,9 @@ void Meter::DrawMeterBar(wxDC &dc, MeterBar *meterBar)
    AColor::Dark(&dc, false);
    for(i=0; i<mNumTicks; i++)
       if (meterBar->vert)
-         dc.DrawLine(r.x+r.width/2-1, mTick[i], r.x+r.width/2+1, mTick[i]);
+         AColor::Line(dc, r.x+r.width/2-1, mTick[i], r.x+r.width/2+1, mTick[i]);
       else
-         dc.DrawLine(mTick[i], r.y+r.height/2-1, mTick[i], r.y+r.height/2+1);
+         AColor::Line(dc, mTick[i], r.y+r.height/2-1, mTick[i], r.y+r.height/2+1);
    */
 
    dc.SetBrush(*wxTRANSPARENT_BRUSH);
@@ -771,29 +1062,47 @@ void Meter::DrawMeterBar(wxDC &dc, MeterBar *meterBar)
 
    if (meterBar->vert) {
       int ht = (int)(meterBar->peakHold * r.height + 0.5);
-      dc.DrawLine(r.x+1, r.y + r.height - ht,
-                  r.x+r.width, r.y + r.height - ht);
+      AColor::Line(dc, r.x+1, r.y + r.height - ht,
+                   r.x+r.width-1, r.y + r.height - ht);
       if (ht > 1)
-         dc.DrawLine(r.x+1, r.y + r.height - ht + 1,
-                     r.x+r.width, r.y + r.height - ht + 1);
+         AColor::Line(dc, r.x+1, r.y + r.height - ht + 1,
+                      r.x+r.width-1, r.y + r.height - ht + 1);
+      dc.SetPen(mPeakPeakPen);
+      ht = (int)(meterBar->peakPeakHold * r.height + 0.5);
+      AColor::Line(dc, r.x+1, r.y + r.height - ht,
+                   r.x+r.width-1, r.y + r.height - ht);
+      if (ht > 1)
+         AColor::Line(dc, r.x+1, r.y + r.height - ht + 1,
+                      r.x+r.width-1, r.y + r.height - ht + 1);
+
+      dc.SetPen(mPen);
       ht = (int)(meterBar->peak * r.height + 0.5);
       r = wxRect(r.x, r.y + r.height - ht,
                  r.width, ht);
-      ht = (int)(meterBar->rms * r.height + 0.5);
-      rRMS = wxRect(r.x, r.y + r.height - ht,
-                    r.width, ht);
+      ht = (int)(meterBar->rms * rRMS.height + 0.5);
+      rRMS = wxRect(rRMS.x, rRMS.y + rRMS.height - ht,
+                    rRMS.width, ht);
+
    }
    else {
       int wd = (int)(meterBar->peakHold * r.width + 0.5);
-      dc.DrawLine(r.x + wd, r.y + 1, r.x + wd, r.y + r.height);
+      AColor::Line(dc, r.x + wd, r.y + 1, r.x + wd, r.y + r.height - 1);
       if (wd > 1)
-         dc.DrawLine(r.x + wd - 1, r.y + 1, r.x + wd - 1, r.y + r.height);
+         AColor::Line(dc, r.x + wd - 1, r.y + 1, r.x + wd - 1, r.y + r.height - 1);
+
+      dc.SetPen(mPeakPeakPen);
+      wd = (int)(meterBar->peakPeakHold * r.width + 0.5);
+      AColor::Line(dc, r.x + wd, r.y + 1, r.x + wd, r.y + r.height - 1);
+      if (wd > 1)
+         AColor::Line(dc, r.x + wd - 1, r.y + 1, r.x + wd - 1, r.y + r.height - 1);
+      
+      dc.SetPen(mPen);
       wd = (int)(meterBar->peak * r.width + 0.5);
       r = wxRect(r.x, r.y,
                  wd, r.height);
-      wd = (int)(meterBar->rms * r.width + 0.5);
-      rRMS = wxRect(r.x, r.y,
-                    wd, r.height);
+      wd = (int)(meterBar->rms * rRMS.width + 0.5);
+      rRMS = wxRect(rRMS.x, rRMS.y,
+                    wd, rRMS.height);
    }
    dc.SetPen(*wxTRANSPARENT_PEN);
    dc.SetBrush(mBrush);
@@ -803,11 +1112,11 @@ void Meter::DrawMeterBar(wxDC &dc, MeterBar *meterBar)
 
    dc.SetBrush(*wxTRANSPARENT_BRUSH);
    dc.SetPen(mLightPen);
-   dc.DrawLine(r.x, r.y, r.x + r.width, r.y);
-   dc.DrawLine(r.x, r.y, r.x, r.y + r.height);
+   AColor::Line(dc, r.x, r.y, r.x + r.width, r.y);
+   AColor::Line(dc, r.x, r.y, r.x, r.y + r.height);
    dc.SetPen(mDarkPen);
-   dc.DrawLine(r.x + r.width, r.y, r.x + r.width, r.y + r.height);
-   dc.DrawLine(r.x, r.y + r.height, r.x + r.width + 1, r.y + r.height);
+   AColor::Line(dc, r.x + r.width, r.y, r.x + r.width, r.y + r.height);
+   AColor::Line(dc, r.x, r.y + r.height, r.x + r.width, r.y + r.height);
 
    if (mClip) {
       if (meterBar->clipping)
@@ -821,16 +1130,24 @@ void Meter::DrawMeterBar(wxDC &dc, MeterBar *meterBar)
    }
 }
 
+bool Meter::IsMeterDisabled() {return mMeterDisabled!=0;}
+
 void Meter::StartMonitoring()
 {
+
    if (gAudioIO->IsMonitoring())
       gAudioIO->StopStream();
    else {
-      gAudioIO->StartMonitoring(44100.0);
+      if (mMeterDisabled){
+         wxCommandEvent dummy;
+         OnDisableMeter(dummy);
+      }
+
       AudacityProject *p = GetActiveProject();
       if (p) {
-         MeterToolBar *bar;
-         bar = p->GetMeterToolBar();
+         gAudioIO->StartMonitoring(p->GetRate());
+
+         MeterToolBar *bar = p->GetMeterToolBar();
          if (bar) {
             Meter *play, *record;
             bar->GetMeters(&play, &record);
@@ -838,12 +1155,62 @@ void Meter::StartMonitoring()
          }
       }
    }
+
 }
 
 //
 // Pop-up menu handlers
 //
 
+void Meter::OnDisableMeter(wxCommandEvent &evt)
+{
+   if (mMeterDisabled) //Enable
+      {
+      mLightPen = mSavedLightPen;
+      mDarkPen = mSavedDarkPen;
+      mBkgndBrush = mSavedBkgndBrush;
+      mBrush = mSavedBrush;
+      mRMSBrush = mSavedRMSBrush;
+
+      mBkgndBrush = mSavedBkgndBrush;
+      mLightPen = mSavedLightPen;
+      Refresh(false);
+
+      mMeterDisabled = false;
+      }
+   else
+      {
+      if (mIsInput) {
+         if (gAudioIO->IsMonitoring())
+            gAudioIO->StopStream();
+      }
+      mSavedLightPen = mLightPen;
+      mSavedDarkPen = mDarkPen;
+      mSavedBkgndBrush = mBkgndBrush;
+      mSavedBrush = mBrush;
+      mSavedRMSBrush = mRMSBrush;
+
+      mLightPen = mDisabledPen;
+      mDarkPen = mDisabledPen;
+      mBkgndBrush = mDisabledBkgndBrush;
+      mBrush = mDisabledBkgndBrush;
+      mRMSBrush = mDisabledBkgndBrush;
+      mLayoutValid = false;
+      Refresh(false);
+
+      mMeterDisabled = true;
+      }
+   if (mIsInput)
+      {
+      gPrefs->Write(wxT("/Meter/MeterInputDisabled"), mMeterDisabled);
+      }
+   else
+      {
+      gPrefs->Write(wxT("/Meter/MeterOutputDisabled"), mMeterDisabled);
+      }
+
+}
+
 void Meter::OnHorizontal(wxCommandEvent &evt)
 {
    SetStyle(HorizontalStereo);
@@ -892,23 +1259,64 @@ void Meter::OnMonitor(wxCommandEvent &evt)
    StartMonitoring();
 }
 
+#ifdef AUTOMATED_INPUT_LEVEL_ADJUSTMENT
+void Meter::OnAutomatedInputLevelAdjustment(wxCommandEvent &evt)
+{
+   if (gAudioIO->AILAIsActive()) {
+      gAudioIO->AILADisable();
+      AudacityProject *p = GetActiveProject();
+      if (p) p->TP_DisplayStatusMessage(_("Automated Input Level Adjustment stopped as requested by user."));
+   }
+   else
+      gAudioIO->AILAInitialize();
+}
+#endif
+
 void Meter::OnFloat(wxCommandEvent &evt)
 {
 }
 
-void Meter::OnDisable(wxCommandEvent &evt)
+void Meter::OnPreferences(wxCommandEvent &evt)
 {
-   ShowWarningDialog(this, "MeterDisableWarning",
-                     _("The VU meters will now be disabled.\n\n"
-                       "To enable them again, open the Preferences dialog and click on the Interface tab."));
+   wxNumberEntryDialog
+      d(this,
+        _("Higher refresh rates make the meter show more frequent\nchanges. A rate of 30 per second or less should prevent\nthe meter affecting audio quality on slower machines."),
+        _("Meter refresh rate per second [1-100]: "),
+        _("Meter Preferences"),
+        mMeterRefreshRate,
+       1,
+       100);
 
-   gPrefs->Write("/GUI/EnableMeterToolBar", false);
-   
-   if(gMeterToolBarStub) {
-      gMeterToolBarStub->UnloadAll();    //Unload all docked METER toolbars
-      delete gMeterToolBarStub;
-      gMeterToolBarStub = NULL;
+   //#if defined(__WXMAC__)
+      // WXMAC doesn't support wxFRAME_FLOAT_ON_PARENT, so we do
+      //
+      // LL:  I've commented this out because if you have, for instance, the meter
+      //      toolbar undocked and large and then you open a dialog like an effect,
+      //      the dialog may appear behind the dialog and you can't move either one.
+      //
+      //      However, I'm leaving it here because I don't remember why I'd included
+      //      it in the first place.
+      // SetWindowClass((WindowRef)d.MacGetWindowRef(), kFloatingWindowClass);
+   //#endif
+
+   if (d.ShowModal() == wxID_OK) {
+      mMeterRefreshRate = d.GetValue();
+      gPrefs->Write(wxT("/Meter/MeterRefreshRate"), mMeterRefreshRate);
    }
-
-   gAudioIO->SetMeters(NULL, NULL);
+   
+   mTimer.Start(1000 / mMeterRefreshRate);
 }
+
+// Indentation settings for Vim and Emacs.
+// Please do not modify past this point.
+//
+// Local Variables:
+// c-basic-offset: 3
+// indent-tabs-mode: nil
+// End:
+//
+// vim: et sts=3 sw=3
+// 
+
+
+
